local addon = LibStub("AceAddon-3.0"):NewAddon("Langoustine","AceEvent-3.0","AceConsole-3.0","LibSink-2.0","AceTimer-3.0", "AceBucket-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Langoustine",false)

local defaults = {
	profile = {
		TankShow=true,
		TankOptions = {
			x=0, 
			y=0, 
			locked=true, 
			unitHeight=15, 
			unitWidth=150, 
			barTexture="Blizzard", 
			growth="Down", 
			icon=true, 
			alpha=1,  
			showAggro=true,
			colorByClass=true,
			scale=1,
			inverseHealth = false,
			enableClickCast = true,
			reverseOrder=false,
			hideAnchor = false,
		},
		MaxTanks = 10, 
		TankShowTarget=true,
		TankShowTargetTarget=true,
		-- minimap/broker support
		Minimap = {},
		-- LibSink for notifications of tank death
		Sink={},
		NotifyMTDeath=true,
		-- Auto Promote and Auto Loot settings
		AutoPromote={},
		AutoPromoteRank = 0,
		AutoLoot={ method="master", threshold="4", looter=""},
		AutoPromoteEnabled=true,
		AutoLootEnabled=true,
		-- Whisper Invites
		WhisperInviteTriggger="invite",
		WhisperInvite=true,
		-- ReadyCheck window state
		ReadyCheck={},
		-- ReloadUI/Crash support for setups
		SetML = false,
		SetLoot = false,
		SetThreshold = false,
	},
}
local guildRanks = {}
local wipe,tinsert = wipe, tinsert
local raiders = {}
-- upcasts of function we wil use a lot
local isRaidOfficer = isRaidOfficer
local GetNumPartyMembers = GetNumPartyMembers
local UnitInRaid = UnitInRaid
local GetNumRaidMembers = GetNumRaidMembers
local ConvertToRaid = ConvertToRaid
local IsInInstance = IsInInstance
local GetRaidRosterInfo = GetRaidRosterInfo
local UnitName = UnitName
local INVITE_BY_LEVEL = L["All level %d or higher characters will be invited to raid in 10 seconds. Please leave your groups."]
local INVITE_BY_RANK = L["All characters of rank %s or higher will be invited to raid in 10 seconds. Please leave your groups."]
-- Timer callbacks
local ltTimer = nil

function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("LangoustineDB", defaults, "Default")
	-- Localize the KeyBindings
	BINDING_HEADER_Langoustine_Tank_List = L["Langoustine"]
	_G["BINDING_NAME_Add Tank"] = L["Add a local tank"]
	_G["BINDING_NAME_Remove Tank"] = L["Remove a local tank"]
	_G["BINDING_NAME_Clear all tanks"] = L["Clear all local tanks"]
	-- End localize	
	self:SetupSlashCommands()
	self.granks = guildRanks
	self:SetSinkStorage(self.db.profile.Sink)
	
end

function addon:SetupSlashCommands()
	self:RegisterChatCommand("radisband", "Disband")
	self:RegisterChatCommand("rapromote", "Promote")
	self:RegisterChatCommand("rainvite",  "MassInvite")
	self:RegisterChatCommand("rademote",  "Demote")
	self:RegisterChatCommand("lang",      "LaunchConfig")
	self:RegisterChatCommand("rawhisper", "Whisper")
end

function addon:Whisper(msg)
	if msg and strlen(msg) > 2 then
		self:Print(string.format(L["Whisper trigger set to %s."],msg))
		self.db.profile.WhisperInvite = true
	else
		if self.db.profile.WhisperInvite then
			self.db.profile.WhisperInvite = false
			self:Print(L["Disabling whisper invites."])
		else
			self.db.profile.WhisperInvite = true
			self:Print(L["Enabling whisper invites."])
		end
	end
end

function addon:LaunchConfig()
	local loaded = LoadAddOn("LangoustineOptions")
	if loaded or self.OptionFrame then
		LibStub("AceConfigDialog-3.0"):Open("Langoustine")
	else
		self:Print(L["Langoustine Options is not installed."])
	end
end

function addon:Demote(player)
	if player then
		if self.db.profile.AutoPromote[player] then
			self.db.profile.AutoPromote[player] = nil
			self:Print(format(L["%s removed from auto promote list."],player))
		end
		if UnitInRaid(player) then
			DemoteAssistant(player)
		end
	else
		self:Print(L["usage: /rademote player"])
	end
end

function addon:Promote(player)
	if player then
		if not self.db.profile.AutoPromote[player] then
			self.db.profile.AutoPromote[player] = true
			self:Print(format(L["%s added to auto promote list."],player))
		end
		if UnitInRaid(player) then
			addon:AutoPromote(player)
		end
	else
		self:Print(L["usage: /rapromote player"])
	end
end
-- when we disable ourselves
function addon:DisableEvents()
	self:UnregisterEvent("CHAT_MSG_WHISPER")
	self.db.profile.SetML = false
	self.db.profile.SetLoot = false
	self.db.profile.SetThreshold = false
end

function addon:OnDisable()
	self:DisableEvents()
end

function addon:OnEnable()
	self:EnableEvents()
	if IsInGuild() then
		self:GuildCheck()
	end
end
-- enable ourselves
function addon:EnableEvents()
	self:RegisterEvent("CHAT_MSG_WHISPER","CheckInvite")
	self:RegisterBucketEvent("RAID_ROSTER_UPDATE",1,"RaidCheck")
	self:RegisterEvent("GUILD_ROSTER_UPDATE","GuildCheck")
	self:RegisterEvent("PLAYER_ENTERING_WORLD","RaidCheck")	
end

function addon:MassInvite(msg)
	local arg = self:GetArgs(msg,1)
	if arg then
		if tonumber(arg) > 0 then
			self:Invite(nil,tonumber(arg))
		elseif strlen(arg) > 2 then
			arg = arg:lower()
			for i,rank in ipairs(guildRanks) do
				if rank == arg then self:Invite(i,nil) end
			end
		end
	else
		self:Print(L["usage: /rainvite level or rank"])
	end
end

-- check whisper invites
function addon:CheckInvite(event, message, player)
	if (IsRaidOfficer()) then
		if self.db.profile.WhisperInvite and message:lower() == self.db.profile.WhisperInviteTriggger then
			local raiders = GetNumRaidMembers()
			if raiders < 40 then
				InviteUnit(player)
			else
				SendChatMessage(L["<Langoustine> The raid is full at this time."], "WHISPER", nil, player)
			end
		end
	end
end
-- update local rank and member list
function addon:GuildCheck()
	wipe(guildRanks)
	for i = 1, GuildControlGetNumRanks() do
		tinsert(guildRanks, GuildControlGetRankName(i):lower())
	end
end

-- conver the party to raid
function addon:ConvertToRaid()
	if GetNumPartyMembers() > 0 and not UnitInRaid("player") then
		self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
		ConvertToRaid()
	end
end

-- disband the raid
function addon:Disband()
	if (IsRaidOfficer()) then
		local player = UnitName("player")
		SendChatMessage(format(L["Raid being disbanded by request of %s "],player),"RAID")
		for i = 1, GetNumRaidMembers() do
			local name = GetRaidRosterInfo(i)
			if name ~= player then
				UninviteUnit(name)
			end
		end
		LeaveParty()
		self:DisableEvents()
	end
end
function addon:FireInvites()
	-- invite everyone
	for _,name in ipairs(raiders) do
		InviteUnit(name)
	end
	wipe(raiders)
end
-- invite by guild rank or by level
function addon:Invite(byRank,byLevel)
	self:RegisterEvent("PARTY_MEMBERS_CHANGED","ConvertToRaid")
	wipe(raiders)
	for i = 1, GetNumGuildMembers() do
		local name, _, rank, level, _, _ , _, _, online = GetGuildRosterInfo(i)
		-- check for both party and raid since we may run invite after grabbign a few people
		if name and online and not UnitInParty(name) and not UnitInRaid(name) then
			if byLevel and level >= byLevel then
				tinsert(raiders, name)
			elseif byRank and (rank + 1) >= byRank then
				tinsert(raiders, name)
			end
		end
	end
	if byRank then
		SendChatMessage(format(INVITE_BY_RANK,guildRanks[byRank]),"GUILD")
	else
		SendChatMessage(format(INVITE_BY_LEVEL,byLevel),"GUILD")
	end
	self:ScheduleTimer("FireInvites",10)
end
function addon:SetLootThreshold()
	local thold = GetLootThreshold()
	if thold ~= tonumber(self.db.profile.AutoLoot.threshold) then
		SetLootThreshold(self.db.profile.AutoLoot.threshold)
	end
	ltTimer = nil
end

function addon:AutoPromote(player)
	for i = 1, GetNumRaidMembers() do
		local name, rank = GetRaidRosterInfo(i)
		if player then
			if name and rank == 0 then
				PromoteToAssistant(name)
			end
		else
			if name and rank == 0 and self.db.profile.AutoPromote[name] then
				PromoteToAssistant(name)
			elseif tonumber(self.db.profile.AutoPromoteRank) > 0 and rank == 0 and name then
				local _,gname,gindex = GetGuildInfo(name)
				if gname and gindex <= tonumber(self.db.profile.AutoPromoteRank) then
					PromoteToAssistant(name)
				end
			end
		end
	end
end
-- Raid Checks
function addon:RaidCheck()
	-- Are we in a raid?
	local inRaid = UnitInRaid("player")
	local _, instanceType = IsInInstance()
	if instanceType == "pvp" or instanceType == "arena" then return end
	-- if we are not enabled and joined a raid enable ourselves
	if inRaid then
		self:EnableEvents()
	elseif not inRaid then -- left the raid group disable ourselves
		self:DisableEvents()
		self:Disable()
		return
	end
	-- Raid leader checks
	if (IsRaidLeader()) then
		-- Check promote list
		if self.db.profile.AutoPromoteEnabled then
			self:AutoPromote()
		end
		-- Check Loot setup
		if self.db.profile.AutoLootEnabled then
			local method,_,masterLooter = GetLootMethod()
			if not self.db.profile.SetLoot then
				-- Change Method
				if method ~= self.db.profile.AutoLoot.method and not self.db.profile.SetLoot then
					if self.db.profile.AutoLoot.method == "master" then
						local mlCandidate = UnitName("player")
						for i = 1, GetNumRaidMembers() do
							local name, rank = GetRaidRosterInfo(i)
							if name == self.db.profile.AutoLoot.looter then
								mlCandidate = name
							end
						end
						SetLootMethod(self.db.profile.AutoLoot.method,mlCandidate)						
						self.db.profile.SetML = false
					else
						SetLootMethod(self.db.profile.AutoLoot.method,self.db.profile.AutoLoot.threshold)
						self.db.profile.SetML = true
					end
				end
				self.db.profile.SetLoot = true
				local thold = GetLootThreshold()
				if thold == tonumber(self.db.profile.AutoLoot.threshold) then
					if ltTimer then
						self:CancelTimer(ltTimer)
					end
				else
					if ltTimer == nil then
						ltTimer = self:ScheduleTimer("SetLootThreshold",3)
					end
				end
			end			
			if method == "master" and self.db.profile.AutoLoot.looter and not self.db.profile.SetML then
				-- do we have the right master looter?
				local looterName = GetRaidRosterInfo(masterLooter)
				if looterName ~= self.db.profile.AutoLoot.looter then
					-- is the looter we want in the raid?
					for i = 1, GetNumRaidMembers() do
						local name, rank = GetRaidRosterInfo(i)
						if name == self.db.profile.AutoLoot.looter then
							SetLootMethod(method,name,self.db.profile.AutoLoot.threshold)
							self.db.profile.SetML = true
							-- we set the ml according to prefs, stop checking anymore till we leave the raid
							-- during a run the raidleader may reset loot as he wants and/or the master looter
						end
					end
				end
			end
		end
	end
end